/**********************************************************************************
	Project for shifting data in and out of a DS1302 Real Time Clock chip

	Based on Demo1302.c by www.maxim-ic.com

	Used PIC16F690 device.

	written by Jess Lewis on 4/20/2011
***********************************************************************************/

#include <htc.h>
#include "DS1302.h"

#define CWPr            0x8E             // Write Protect Register
#define WrBurst         0xBE             // Write Burst Of Data

#define DS1302_CE_DDR 	    TRISC5		//select direction register
#define DS1302_IO_DDR 	    TRISA0		//input/output data direction register SDIN
#define DS1302_SCK_DDR 		TRISA5		//clock direction register       SCLK

#define DS1302_CE_PORT 		RC5		//select port
#define DS1302_IO_PORT	    RA0  	//input/output data port 
#define DS1302_SCK_PORT 	RA5		//clock port


/**************************************************************************************
 * void DS1302ChangeTime(unsigned char selecteddigit)
 *
 * changes time values 
 **************************************************************************************/
void DS1302ChangeDT(unsigned char selecteddigit)
{
	unsigned char templo, temphi;

	switch(selecteddigit)
	{
		case 0: 
			//month
			timeArray[4] = timeArray[4] + 0x01;
			templo = timeArray[4] & 0x0F; 		//get low bits		
			temphi = timeArray[4] & 0x10; 	//get hi bits
			if ((templo > 2) & (temphi == 0x10)) timeArray[4] = 0x01;
			if (templo > 9) timeArray[4] = 0x10;
			break;
		case 1:	
			//date
			timeArray[3] = timeArray[3] + 0x01;
			templo = timeArray[3] & 0x0F; 		//get low bits		
			temphi = timeArray[3] & 0x30; 	//get hi bits
			if ((templo > 1) & (temphi == 0x30)) timeArray[3] = 0x01;
			if (templo > 9) 
			{
				timeArray[3] = timeArray[3] & 0x30;
				timeArray[3] = timeArray[3] + 0x10;
			}
			break;
		case 2:		//year			
			timeArray[6] = timeArray[6] + 0x01;
			templo = timeArray[6] & 0x0F; 		//get low bits		
			temphi = timeArray[6] & 0xF0; 	//get hi bits
			if ((templo > 0) & (temphi == 0x50)) timeArray[6] = 0x01;
			if (templo > 9) 
			{
				timeArray[6] = timeArray[6] & 0xF0;
				timeArray[6] = timeArray[6] + 0x10;
			}
			break; 
		case 3:		//hours
			timeArray[2] = timeArray[2] + 0x01;
			templo = timeArray[2] & 0x0F; 		//get low bits		
			temphi = timeArray[2] & 0x30; 	//get hi bits
			if ((templo > 3) & (temphi == 0x20)) timeArray[2] = 0x00;
			if (templo > 9) 
			{
				timeArray[2] = timeArray[2] & 0x30;
				timeArray[2] = timeArray[2] + 0x10;
			}
			break;
		case 4:		//mins
			timeArray[1] = timeArray[1] + 0x01;	
			templo = timeArray[1] & 0x0F; 		//get low bits		
			temphi = timeArray[1] & 0x70; 	//get hi bits
			if ((templo > 9) & (temphi == 0x50)) timeArray[1] = 0x00;
			if ((templo > 9) & (timeArray[1] != 0x00))
			{
				timeArray[1] = timeArray[1] & 0x70;
				timeArray[1] = timeArray[1] + 0x10;
			}	
			break;
		case 5:	//secs
			timeArray[0] = timeArray[0] + 0x01;	
			templo = timeArray[0] & 0x0F; 		//get low bits		
			temphi = timeArray[0] & 0x70; 	//get hi bits
			if ((templo > 9) & (temphi == 0x50)) timeArray[0] = 0x00;
			if ((templo > 9) & (timeArray[0] != 0x00))
			{
				timeArray[0] = timeArray[0] & 0x70;
				timeArray[0] = timeArray[0] + 0x10;
			}	
			break;
	}

	setTimeDate_DS1302();
}


/**************************************************************************************
 * void DS1302Config_IO
 *
 * configures I/O and makes sure that all analog ports are set to digital 
 **************************************************************************************/
void DS1302Config_IO(void)
{
	//all analogs set to digital and all comparators turned off in shiftdata code
	//otherwise it would need to be done for DS1302 to work
		
	DS1302_CE_DDR = 0; 	//select direction register
	DS1302_IO_DDR = 0;     //may be an input or output	    
	DS1302_SCK_DDR = 0;	//clock signal is always an output
}


/**************************************************************************************
 * void read_DS1302TimeDate(void)
 *
 * reads time and date
 **************************************************************************************/
void read_DS1302TimeDate(void)
{
	unsigned char yr1, mn1, date1, dy1, hr1, min1, sec1;
	
	reset_3w();
	wbyte_3w(0xBF); /* clock burst */
	sec1 = rbyte_3w();
	min1 = rbyte_3w();
	hr1 = rbyte_3w();
	date1 = rbyte_3w();
	mn1 = rbyte_3w();
	dy1 = rbyte_3w();
	yr1 = rbyte_3w();
	reset_3w();

	timeArray[0] = sec1;
	timeArray[1] = min1;
	timeArray[2] = hr1;
	timeArray[3] = date1;
	timeArray[4] = mn1;
	timeArray[5] = dy1;
	timeArray[6] = yr1;

	//for the tens position we need the top 1 bit (month tens digit is zero or 1) 
	// and then shift them right 4 to display as a digit
	//for the ones position we need the bottom 4 bits although the count is only 0-9 (BCD)
	datetimestring[0] = mn1 & 0x10;
	datetimestring[0] = datetimestring[0] >>4;
	datetimestring[1] = mn1 & 0x0f;

	//for the tens position we need the top 2 bits (0,1,2, or 3 is the tens digit )
	// and then shift them right 4 to display as a digit
	//for the ones position we need the bottom 4 bits although the count is only 0-9 (BCD)
	datetimestring[2] = date1 & 0x30;
	datetimestring[2] = datetimestring[2] >>4;
	datetimestring[3] = date1 & 0x0f;
	
	datetimestring[4] = 2; //record thousands digit - it's 2000+
	datetimestring[5] = 0;  //record hundreds digit

	//for the tens position we need the top 4 bits and then shift them right 4 to display as a digit
	//for the ones position we need the bottom 4 bits although the count is only 0-9 (BCD)
	datetimestring[6] = yr1 & 0xf0;
	datetimestring[6] = datetimestring[6] >>4;
	datetimestring[7] = yr1 & 0x0f;

	//for the tens position we need the top 2 bits (0,1, or 2 is the hour)
	// and then shift them right 4 to display as a digit
	//for the ones position we need the bottom 4 bits although the count is only 0-9 (BCD)
	datetimestring[8] = hr1 & 0x30;
	datetimestring[8] = datetimestring[8] >>4;
	datetimestring[9] = hr1 & 0x0f;

	//for the tens position we need the top 3 bits (0-5 is the minute)
	// and then shift them right 4 to display as a digit
	//for the ones position we need the bottom 4 bits although the count is only 0-9 (BCD)
	datetimestring[10] = min1 & 0x70;
 	datetimestring[10] = datetimestring[10] >>4;
	datetimestring[11] = min1 & 0x0f;

	datetimestring[12] = sec1 & 0x70; 
	datetimestring[12] = datetimestring[12] >>4;
	datetimestring[13] = sec1 & 0x0f; 

	for(char t=0; t<14; t++)
	{
		if(datetimestring[t] < 1)   //((datetimestring[t] < 1) || (datetimestring[t] > 10))
		{
			datetimestring[t] = 10;  //MAX7456 OSD chip uses 0x0A as zero digit instead of zero
		}
	}
}


/**************************************************************************************
 * void initialize_DS1302(void)
 *
 * initializes the DS1302 
 **************************************************************************************/
void initialize_DS1302(void)
{
	reset_3w();
	wbyte_3w(CWPr); /* control register */
	wbyte_3w(0); /* disable write protect */
	reset_3w();
	
}

/**************************************************************************************
 * void setTimeDate_DS1302(void)
 *
 * sets the time and date
 **************************************************************************************/
void setTimeDate_DS1302(void)
{
	unsigned char hrs, temp;

	//set for 24 hour clock
	hrs = timeArray[2] & 0x3f;

	reset_3w();
	wbyte_3w(WrBurst); /* clock burst write (eight registers) */
	wbyte_3w(timeArray[0]); 	//sec
	wbyte_3w(timeArray[1]); 	//min
	wbyte_3w(hrs);				//hr
	wbyte_3w(timeArray[3]); 	//date
	wbyte_3w(timeArray[4]);		//month
	wbyte_3w(timeArray[5]); 	//day
	wbyte_3w(timeArray[6]);		//year
	wbyte_3w(0); /* must write control register in burst mode */
	reset_3w();
}


/**************************************************************************************
 * void reset_3w(void)
 *
 * resets the DS1302
 **************************************************************************************/
void reset_3w(void)
{
	DS1302_SCK_PORT = 0;
	DS1302_CE_PORT  = 0;
	DS1302_CE_PORT  = 1;
}


/**************************************************************************************
 * void wbyte_3w(unsigned char W_Byte)
 *
 * Writes to an 8-bit register with the SPI port (shiftout LSBFIRST)
 **************************************************************************************/
void wbyte_3w(unsigned char W_Byte) 
{
	unsigned char i;

	DS1302_IO_DDR = 0; //set to output
	for(i = 0; i < 8; ++i)
	{
		DS1302_IO_PORT = 0;
		if(W_Byte & 0x01)
		{
			DS1302_IO_PORT = 1; /* set port pin high to read data */
		}
		DS1302_SCK_PORT = 0;
		DS1302_SCK_PORT = 1;
		W_Byte >>= 1;
	}
}


/**************************************************************************************
 * unsigned char rbyte_3w(void)
 *
 * Reads an 8-bit register with the SPI port.
 * Data is returned.  
 **************************************************************************************/
unsigned char rbyte_3w(void)
{
	unsigned char i;
	unsigned char R_Byte;
	unsigned char TmpByte;

	DS1302_IO_DDR = 1; //set to input

	R_Byte = 0x00;
	DS1302_IO_PORT = 1;
	for(i=0; i<8; ++i)
	{
		DS1302_SCK_PORT = 1;
		DS1302_SCK_PORT = 0;
		TmpByte = (unsigned char)DS1302_IO_PORT;
		TmpByte <<= 7;
		R_Byte >>= 1;
		R_Byte |= TmpByte;
	}

	DS1302_IO_DDR = 0; //set to input
	return R_Byte;
}
