/* Project for shifting data in and out of a MAX7456 OSC chip
	and other SPI type devices.
 * Used PIC16F688 device. */

#include <htc.h>
#include "shiftdata.h"

#define CS_DDR 		TRISA2		//select direction register
#define MOSI_DDR 	TRISC0		//output data direction register SDIN
#define MISO_DDR 	TRISC2		//input data direction register  SDOUT
#define SCK_DDR 	TRISC1		//clock direction register       SCLK

#define CS_PORT 	RA2		//select port
#define MOSI_PORT 	RC0  	//output data port 
#define MISO_PORT 	RC2		//input data port
#define SCK_PORT 	RC1		//clock port

/**************************************************************************************
 * spiWriteReg
 *
 * Writes to an 8-bit register with the SPI port
 **************************************************************************************/

void spiWriteReg(unsigned char address, unsigned char byte)
{
	unsigned char SPICount;                                       // Counter used to clock out the data

	unsigned char SPIData;                                        // Define a data structure for the SPI data
	
	CS_PORT = 1;                                        		// Make sure we start with active-low CS high
	SCK_PORT = 0;                                       		// and CK low
	//__delay_ms(1);
	
	SPIData = address;                                            // Preload the data to be sent with Address
	CS_PORT = 0;                                           // Set active-low CS low to start the SPI cycle 

	for (SPICount = 0; SPICount < 8; SPICount++)                  // Prepare to clock out the Address byte
	{
		if (SPIData & 0x80) 
			MOSI_PORT = 1;           				// Check for a 1 and set the MOSI line appropriately
		else 
			MOSI_PORT = 0;
		
		SCK_PORT = 1;                                         // Toggle the clock line 1 0
		SCK_PORT = 0;

		SPIData <<= 1;                                              // Rotate to get the next bit
	}                                                             // and loop back to send the next bit											
  															// Repeat for the Data byte
	SPIData = byte;                                            // Preload the data to be sent with Data
	
	for (SPICount = 0; SPICount < 8; SPICount++)
	{
		if (SPIData & 0x80) 
			MOSI_PORT = 1;
		else 
			MOSI_PORT = 0;
		
		SCK_PORT = 1;                                         // Toggle the clock line 1 0
		SCK_PORT = 0;

		SPIData <<= 1; 
	}           
	
	CS_PORT  = 1;
	MOSI_PORT = 0;
}


/**************************************************************************************
 * spiWriteRegAutoIncr
 *
 * Writes to an 8-bit register with the SPI port
 * using the MAX7456's auto-increment mode
 **************************************************************************************/

void spiWriteRegAutoIncr(unsigned char regData)
{
  unsigned char SPICount;                               // Counter used to clock out the data
  unsigned char SPIData;                                // Define a data structure for the SPI data.
  
  CS_PORT = 1;                                           // Make sure we start with /CS high
  SCK_PORT = 0;                                           // and CK low
//  __delay_ms(1);

  SPIData = regData;                                    // Preload the data to be sent with Address & Data

  CS_PORT = 0;                                           // Set /CS low to start the SPI cycle
  
  for (SPICount = 0; SPICount < 8; SPICount++)          // Prepare to clock out the Address & Data
  {
    if (SPIData & 0x80)
      MOSI_PORT = 1;
    else
      MOSI_PORT = 0;
    
	SCK_PORT = 1;
	SCK_PORT = 0;
    SPIData <<= 1;
  }                                                     // and loop back to send the next bit   
  MOSI_PORT = 0;                                         // Reset the MOSI data line
}


/**************************************************************************************
 * spiReadReg
 *
 * Reads an 8-bit register with the SPI port.
 * Data is returned. 
 **************************************************************************************/

unsigned char spiReadReg(unsigned char address)
{
	unsigned char SPICount;                                       // Counter used to clock out the data
	unsigned char SPIData; 
		
	CS_PORT = 1;                                        		// Make sure we start with active-low CS high
	SCK_PORT = 0;                                       		// and CK low

	SPIData = address;                                            // Preload the data to be sent with Address
	CS_PORT = 0;                                           // Set active-low CS low to start the SPI cycle 
	
	for (SPICount = 0; SPICount < 8; SPICount++)                  // Prepare to clock out the Address byte
	{
		if (SPIData & 0x80) 
			MOSI_PORT = 1;           				// Check for a 1 and set the MOSI line appropriately
		else 
			MOSI_PORT = 0;
		
		SCK_PORT = 1;                                         // Toggle the clock line
		SCK_PORT = 0;	
		
		SPIData <<= 1;                                              // Rotate to get the next bit
	}                                                             // and loop back to send the next bit	

	MOSI_PORT = 0;                                         // Reset the MOSI data line
	
	SPIData = 0;
	
	for (SPICount = 0; SPICount < 8; SPICount++)                  // Prepare to clock in the data to be read
	{
		SPIData <<=1;                                               // Rotate the data
		SCK_PORT = 1;                                        // Raise the clock to clock the data out of the MAX7456
		
		SPIData += MISO_PORT;                                // Read the data bit
		SCK_PORT = 0;                                       // Drop the clock ready for the next bit
	}                                                             // and loop back
	CS_PORT = 1;                                                 // Raise CS
					  
	return SPIData;                              // Finally return the read data
}


/**************************************************************************************
 * configure I/O
 *
 * configures I/O and makes sure that all analog ports are set to digital 
 **************************************************************************************/

void shiftdataconfig_io (void)
{
	//set comparators to digital i/o must be done
	CMCON0 = 0x07;

	//set analogs to digital i/o must be done	
  	ANSEL = 0; 
		
	CS_DDR = 0;    //select direction register
	MOSI_DDR = 0;  //output data direction register
	MISO_DDR = 1;  //input data direction register
	SCK_DDR = 0;   //clock direction register

	
}
