#include <htc.h>
#include <stdio.h>
#include "timing.h"

volatile near unsigned int newmillisecond;

void delayMs(unsigned int x){ 
	unsigned long start_millis = 0;
	unsigned long end_millis = 0;

	//limited to 999 milliseconds
	if(x > 999) 
	{
		x = 999;
	}

	start_millis = newmillisecond;  
	end_millis = start_millis + x;

	//end_millis needs to be limited to a full second
	if (end_millis > 999)
	{
		end_millis = end_millis - 999;
	}

	//if the counter wraps around then we count until the counter is less than the start
	// or in other words until we overflow and then count up until the counter is greater than 
	// end otherwise a normal count
	if (end_millis < start_millis)
	{
		//adjustment for handling wrapping around
		while(newmillisecond > start_millis)
		{
			continue;
		} 		
	}
	
	//gotta do this whether there is a wraparound or not
	while(newmillisecond < end_millis)
	{
		continue;
	}				
}

// Here be interrupt function - the name is unimportant.
void timingisr(void){

	if((T0IE) && (T0IF)) 
	{	
		//frequency is 976.5625hz or 1 pulse every 1.02msec
		
		newmillisecond++;  //a millisecond has accumulated	
	
		if(newmillisecond > 999)
		{			
			RTCIntISR(); //update real-time clock
			newmillisecond = 0; //reset the milliseconds
		}
				
		TMR0 = TIMER0PRESET;  //fine tuning	
		T0IF = 0;			// clear the interrupt flag
	}  //END if((T0IE) && (T0IF)) 
}

void initTiming_interrupts(void){

	//	Timer 0 interrupt enable 
	//  clock frequency/4/prescaler/max count = interrupt frequency in hertz
	//  example:  4,000,000/4/4/256 = 976.5625hz
	//  duration between interrupts:  1/hz or in this case 1/976.562hz = 0.001024 or 1.02msec
	//  see page 232 of "Designing Embedded Systems with PIC Microcontrollers" by Tim Wilmshurst
	T0CS = 0; 	// timer0 clock source set to internal clock
	PSA = 0; 	// prescaler is assigned to timer0
	
	PS2 = 0;	//prescaler set to 1:4
	PS1 = 0;
	PS0 = 1;

	TMR0 = TIMER0PRESET;  //fine tuning
	T0IF = 1;  //clear the interrupt flag
	T0IE = 1;  // timer 0 interrupt enable
}

//////////////////////////////////////////////////////////////////////////


/****************************************************************************
 Function: RTCIntInit

 PreCondition: None

 Overview:
       This routine is used for RTC Initialization
       It initializes according to compile time selections.

 Input: None


 Output: None

	NOTE:  code from this routine may be copied to an existing initinterrupts routine in
			which case it is not necessary to call this routine.  Instead call RTCChangeDateTime.
****************************************************************************/
void RTCIntInit(void){

	vMonth = eeprom_read(0x0000);
	vDay = eeprom_read(0x0001);
	vYear = eeprom_read(0x0002);
	vHour = eeprom_read(0x0003);
	vMin = eeprom_read(0x0004);
	vSec = eeprom_read(0x0005);
}


/****************************************************************************
 Function: RTCIntISR

 PreCondition: None

 Overview:
       This is a Interrupt service routine for Timer1 Interrupt.
       It handles updating the RTC.
       Call it from Interrupt service routine if an interrupt was caused by
       Timer1.

 Input:


 Output:   Updates the Real time clock (sec,min,hour,day,dow,month,year,
           century, and vRTCStatus which contains the flags that are set
           when one of the variables in RTC changes) .

****************************************************************************/
void RTCIntISR(void){

	unsigned char daysforcurrentmonth = 0; //of course it's 28,29,30 or 31 but only after initializing
	unsigned char daysTable[]={29, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

	vSec ++; //increment the seconds (vSec++)
	if(vSec < 60)  
	{
		return;  //no need to rollover seconds so exit
	}

	vSec = 0;  //else clear the seconds vSec = 00
	vMin ++;   //increment the minutes
	if (vMin < 60)
	{
		return;  //no need to rollover minutes so exit
	}

	vMin = 0; //else clear the Minutes vMin=00
	vHour ++;  //increment the hours
	if(vHour < 24)
	{
		return;  //no need to rollover hours so exit
	}
	
	vHour = 0;  //else clear the hours vHour=00
	vDay ++;   //increment the days
	
   	if (((vYear && 0b00000011) == 0) && (vMonth == 2)) //if the 2-LSbs of vYear are zero then this is a leap year
	{
    	daysforcurrentmonth = daysTable[0];
    }
	else
	{
		//ignore leap year here
		daysforcurrentmonth = daysTable[vMonth];
	}

	//check for new month
	if(vDay <= daysforcurrentmonth)
	{
		return;
	}
	
	vDay = 1;
	vMonth ++; //add a month 

	if(vMonth < 13)
	{
		return; //no need to rollover months so exit
	}

	vMonth = 1; //month rolled over
	vYear ++;  //it's a new year!
}  //end RTCIntISR

/****************************RTCcurrenttimedatestring **********
 purpose:  sets up a character array with data from current time and date
			to be used for display

inputs:  none as variables used are global - otherwise time and date
output:  writes data to character array that is global

	datetimestring[0] = '0'; //month tens
	datetimestring[1] = '1'; //month ones

	datetimestring[2] = '0'; //date tens
	datetimestring[3] = '1'; //date ones

	datetimestring[4] = '2'; //year thousands
	datetimestring[5] = '0'; //year hundreds
	datetimestring[6] = '1'; //year tens
	datetimestring[7] = '1'; //year ones

	datetimestring[8] = '0'; //hours tens
	datetimestring[9] = '1'; //hours ones
	
	datetimestring[10] = '0'; //minutes tens
	datetimestring[11] = '1'; //minutes ones

	datetimestring[12] = '0'; //seconds tens
	datetimestring[13] = '1'; //seconds ones

***********************************************************/
void RTCcurrenttimedatestring(void){
	unsigned char hundigit, tendigit, onedigit;

	datetimestring[0] = vMonth / 10;  //month tens digit
	tendigit = datetimestring[0] * 10;
	datetimestring[1] = vMonth - tendigit; //record ones digit for months

	datetimestring[2] = vDay / 10;  //day tens digit
	tendigit = datetimestring[2] * 10;
	datetimestring[3] = vDay - tendigit;  //record ones digit for days

	datetimestring[4] = 2; //record thousands digit - it's 2000+
	datetimestring[5] = vYear / 100;  //record hundreds digit
	hundigit = datetimestring[5] * 100;
	datetimestring[6] = (vYear - hundigit) / 10; //record tens digit
	tendigit = datetimestring[6] * 10;  
	datetimestring[7] = vYear - hundigit - tendigit; //record ones digit

	datetimestring[8] = vHour / 10; //record tens digit for hours
	tendigit = datetimestring[8] * 10;
	datetimestring[9] = vHour - tendigit; //record ones digit for hours

	datetimestring[10] = vMin / 10;  //record tens digit for minutes
	tendigit = datetimestring[10] * 10;
	datetimestring[11]  = vMin - tendigit;  //record ones digit for minutes

	datetimestring[12] = vSec / 10;  //record tens digit for seconds
	tendigit = datetimestring[12] * 10;
	datetimestring[13] = vSec - tendigit;  //record ones digit for seconds

	for(char t=0; t<14; t++)
	{
		if((datetimestring[t] < 1) || (datetimestring[t] > 10))
		{
			datetimestring[t] = 10;  //MAX7456 OSD chip uses 0x0A as zero digit instead of zero
		}
	}
}


void RTCWriteDateTime(void){

	eeprom_write(0x0000, vMonth);
	eeprom_write(0x0001, vDay);
	eeprom_write(0x0002, vYear);
	eeprom_write(0x0003, vHour);
	eeprom_write(0x0004, vMin);
	eeprom_write(0x0005, vSec);
}

//changing minutes instead of seconds because it could take an eternity to scroll forward
//a day or so if incrementing by seconds.
void RTCIncTime(void){
	
	vMin ++;
	if (vMin < 60)
	{
		return;  //no need to rollover minutes so exit
	}

	vMin = 0; //else clear the Minutes vMin=00
	vHour ++;  //increment the hours
	if(vHour < 24)
	{
		return;  //no need to rollover hours so exit
	}

	vHour = 0;  //else clear the hours vHour=00
	vDay ++;   //increment the days
}


void RTCDecTime(void){
	
	/* since the time variables are all unsigned, they will never go less than zero
	   however, they can wrap around to 255, thus if vMin is greater than 59 then
	   it has in fact rolled over and is regarded as negative
	*/

	vMin --;
	if (vMin < 60)
	{
		return;  //no need to rollover minutes so exit
	}

	vMin = 0; //else clear the Minutes vMin=00
	vHour --;  //decrement the hours
	if(vHour < 24)
	{
		return;  //no need to rollover hours so exit
	}

	vHour = 0;  //else clear the hours vHour=00
	vDay --;   //decrement the days
}
